/******************************************************************************
Filename:   at_mega_8515.h
Filetype:   C header file
Author:     Ray Hurst
Created:    3/10/2008
Edited:     3/14/2008

Description:    C header file for the Atmel AVR ATmega8515 Microcontroller
******************************************************************************/

#ifndef _AVR_ATMEGA8515_H_
#define _AVR_ATMEGA8515_H_ 1

#define _EMBEDDED_CLASS_    1
#ifdef _EMBEDDED_CLASS_

#define cli()  __asm__ __volatile__ ("cli" ::)
#define sei()  __asm__ __volatile__ ("sei" ::)

typedef unsigned char uint8_t;
typedef unsigned int  uint16_t;

#define _MMIO_BYTE(mem_addr)    (*(volatile uint8_t *)(mem_addr))   /* macro for  8 bit general memory access */
#define _MMIO_WORD(mem_addr)    (*(volatile uint16_t *)(mem_addr))  /* macro for 16 bit general memory access */

#define _SFR_IO8(io_addr)       _MMIO_BYTE((io_addr) + 0x20)        /* macro for  8 bit register access */
#define _SFR_IO16(io_addr)      _MMIO_WORD((io_addr) + 0x20)        /* macro for 16 bit register access */

#define _BV(bit)                (1 << (bit))
#define _VECTOR(N)              __vector_ ## N                      /* macro for interrupt vectors */

#if __GNUC__ >= 4 && __GNUC_MINOR__ >= 1
#  define __INTR_ATTRS used, externally_visible
#else /* GCC < 4.1 */
#  define __INTR_ATTRS used
#endif

#define ISR(vector, ...)            \
    void vector (void) __attribute__ ((signal,__INTR_ATTRS)) __VA_ARGS__; \
    void vector (void)

#  define ISR_NOBLOCK    __attribute__((interrupt))

#else   /* !_EMBEDDED_CLASS_ */

/* This file should only be included from <avr/io.h>, never directly. */

  #ifndef _AVR_IO_H_
    #  error "Include <avr/io.h> instead of this file."
  #endif

  #ifndef _AVR_IOXXX_H_
    #  define _AVR_IOXXX_H_ "at_mega_8515.h"
  #else
    #  error "Attempt to include more than one <avr/ioXXX.h> file."
  #endif 
#endif   /* _EMBEDDED_CLASS_ */


/*---------------------------------------------------------------------------*/
/* Oscillator Calibration Register */
#define OSCCAL  _SFR_IO8(0x04)


/*---------------------------------------------------------------------------*/
/* Analog Comparator Control and Status Register */
#define ACSR    _SFR_IO8(0x08)

#define    ACD          7       /* Analog Comparator Disable, 1 = disable */
#define    ACBG         6       /* Analog Comparator Bandgap Select, 1 = internal, 0 = external */
#define    ACO          5       /* Analog Comparator Output */
#define    ACI          4       /* Analog Comparator Interrupt Flag */
#define    ACIE         3       /* Analog Comparator Interrupt Enable, 1 = enable */
#define    ACIC         2       /* Analog Comparator Input Capture Enable, 1 = enable */
#define    ACIS1        1       /* Analog Comparator Interrupt Mode Select bit 1 */
#define    ACIS0        0       /* Analog Comparator Interrupt Mode Select bit 0 */

#define ACSR_MODE_TOGGLE    (0)                         /* analog comparator interrupt on output toggle */
#define ACSR_MODE_FE        _BV(ACIS1)                  /* analog comparator interrupt on output falling edge */
#define ACSR_MODE_RE        (_BV(ACIS1) | _BV(ACIS0))   /* analog comparator interrupt on output rising edge */


/*---------------------------------------------------------------------------*/
/* USART - Universal Synchronous and Acychronous serial Receiver and Transmitter Registers */

#define UBRRL   _SFR_IO8(0x09)  /* UART Baud Rate Register (LSB) */
#define UCSRB   _SFR_IO8(0x0A)  /* USART Control and Status Register B */
#define    RXCIE        7       /* RX Complete Interrupt Enable */
#define    TXCIE        6       /* TX Complete Interrupt Enable */
#define    UDRIE        5       /* Data Register Empty Interrupt Enable */
#define    RXEN         4       /* Receiver Enable */
#define    TXEN         3       /* Transmitter Enable */
#define    UCSZ2        2       /* UCSZ bit 2 - set for 9 bit character */
#define    RXB8         1       /* Receive  Data Bit 8 */
#define    TXB8         0       /* Transmit Data Bit 8 */

#define UCSRA   _SFR_IO8(0x0B)  /* USART Control and Status Register A */
#define    RXC          7       /* Receive Complete  */
#define    TXC          6       /* Transmit Complete */
#define    UDRE         5       /* Data Register Empty */
#define    FE           4       /* Frame Error */
#define    DOR          3       /* Data OverRun */
#define    PE           2       /* Parity Error */
#define    U2X          1       /* Double the Transmission Speed */
#define    MCPM         0       /* Multi-processor Communication mode */

#define UDR	    _SFR_IO8(0x0C)  /* USART I/O Data Register */

#define UBRRH   _SFR_IO8(0x20)  /* USART Baud Rate Register HI         */
#define UCSRC   UBRRH           /* USART Control and Status Register C */

/* USART Control and Status Register C */
#define    URSEL        7       /* USART Register Select, 0 = select UBRRH, 1 = select UCSRC regsiter */
#define    UMSEL        6       /* USART Mode Select, 0 = asynchronous, 1 = synchronous */
#define    UPM1         5       /* USART Parity Mode bit 1 */
#define    UPM0         4       /* USART Parity Mode bit 0 */
#define    USBS         3       /* USART Stop Bit Select */
#define    UCSZ1        2       /* USART Character Size bit 1 */
#define    UCSZ0        1       /* USART Character Size bit 0 */
#define    UCPOL        0       /* USART Clock Polarity, 0 for asychronous mode */
                                /*   Synchronous Mode
                                    UCPOL   Transmitted Data Changed  Receive Data Sampled
                                      0      rising  XCK edge           falling XCK edge
                                      1      falling XCK edge           rising  XCK edge */

#define USART_REG_SELECT_UBBRH          (0)
#define USART_REG_SELECT_UCSRC          _BV(URSEL)

#define USART_MODE_SELECT_ASYNCHRONOUS  (0)
#define USART_MODE_SELECT_SYNCHRONOUS   _BV(UMSEL)

#define USART_PARITY_MODE_NONE         (0)
#define USART_PARITY_MODE_EVEN          _BV(UPM1)
#define USART_PARITY_MODE_ODD           (_BV(UPM1) | _BV(UPM0))

#define USART_STOP_BITS_1               (0)
#define USART_STOP_BITS_2               _BV(USBS)

#define USART_CHAR_SIZE_BITS_5          (0)
#define USART_CHAR_SIZE_BITS_6          _BV(UCSZ0)
#define USART_CHAR_SIZE_BITS_7          _BV(UCSZ1)
#define USART_CHAR_SIZE_BITS_7          _BV(UCSZ1)
#define USART_CHAR_SIZE_BITS_8_OR_9     (_BV(UCSZ1) | _BV(UCSZ0))   /* 9 bit character - must set CHR9 in UCR register */


/*---------------------------------------------------------------------------*/
/* Serial Peripheral Interface (SPI) Registers */
 
#define SPCR    _SFR_IO8(0x0D)  /* SPI Control Register */

#define    SPIE         7   /* SPI Interrupt Enable, 1 = enable */
#define    SPE          6   /* SPI Enable, 1 = enable */
#define    DORD         5   /* SPI Data Order, 1 = MSbit first, 0 = LSBit first */
#define    MSTR         4   /* SPI Master/Slave Select, 1 = master, 0 = slave */
#define    CPOL         3   /* SPI Clock Polarity */
                            /*   1 = SCK high when idle, 0 = SCK low when idle
                                 CPOL     Leading Edge    Trailing Edge
                                  0         Rising          Falling
                                  1         Falling         Rising */
#define    CPHA         2   /* SPI Clock Phase */
                            /*   CPHA     Leading Edge    Trailing Edge
                                  0         Sample          Setup
                                  1         Setup           Sample */
#define    SPR1         1   /* SPI Clock Rate Select bit 1 */
#define    SPR0         0   /* SPI Clock Rate Select bit 0 */

/* SCK frequency select */
/* Following defines for SPSR register - SPI2X bit must be 0 */
#define SPCR_SPR_DIV_4          (0)                         /* Fosc divided by 4 */
#define SPCR_SPR_DIV_16         _BV(SPR0)                   /* Fosc divided by 16 */
#define SPCR_SPR_DIV_64         _BV(SPR1)                   /* Fosc divided by 64 */
#define SPCR_SPR_DIV_128        (_BV(SPR1) | _BV(SPR0))     /* Fosc divided by 64 */
/* Following defines for SPSR register - SPI2X bit must be 1 */
#define SPCR_SPR_DIV_2          (0)                         /* Fosc divided by 2 */
#define SPCR_SPR_DIV_8          _BV(SPR0)                   /* Fosc divided by 8 */
#define SPCR_SPR_DIV_32         _BV(SPR1)                   /* Fosc divided by 32 */


#define SPSR    _SFR_IO8(0x0E)  /* SPI Status Register */

#define    SPIF         7       /*   SPI Interrupt Flag */
#define    WCOL         6       /*   Write Collision Flag, 1 = collision occured */
#define    SPI2X        0       /*  Double SPI Speed, 1 = double SCK frequency */ 


#define SPDR    _SFR_IO8(0x0F)  /* SPI I/O Data Register */


/*---------------------------------------------------------------------------*/
/* PORT A - E Registers */

/* PORT E register */
#define PINE    _SFR_IO8(0x05)  /* Input Pins, Port E */
#define DDRE    _SFR_IO8(0x06)  /* Data Direction Register, Port E */
#define PORTE   _SFR_IO8(0x07)  /* Data Register, Port E */

#define    PE2          2       /* Data Register, Port E pin 2 */
#define    PE1          1       /* Data Register, Port E pin 1 */
#define    PE0          0       /* Data Register, Port E pin 0 */

#define    DDE2         2       /* Data Direction Register, Port E pin 2, 0 = input, 1 = output */
#define    DDE1         1       /* Data Direction Register, Port E pin 1, 0 = input, 1 = output */
#define    DDE0         0       /* Data Direction Register, Port E pin 0, 0 = input, 1 = output */


/* PORT D register */
#define PIND	_SFR_IO8(0x10)  /* Input Pins, Port D */
#define DDRD	_SFR_IO8(0x11)  /* Data Direction Register, Port D */
#define PORTD	_SFR_IO8(0x12)  /* Data Register, Port D */

#define    PD7          7   /* Data Register - Port D bit 7 */
#define    PD6          6   /* Data Register - Port D bit 6 */
#define    PD5          5   /* Data Register - Port D bit 5 */
#define    PD4          4   /* Data Register - Port D bit 4 */
#define    PD3          3   /* Data Register - Port D bit 3 */
#define    PD2          2   /* Data Register - Port D bit 2 */
#define    PD1          1   /* Data Register - Port D bit 1 */
#define    PD0          0   /* Data Register - Port D bit 0 */

#define    DDD7         7   /* Data Direction Register, Port D bit 7, 0 = input, 1 = output */
#define    DDD6         6   /* Data Direction Register, Port D bit 6, 0 = input, 1 = output */
#define    DDD5         5   /* Data Direction Register, Port D bit 5, 0 = input, 1 = output */
#define    DDD4         4   /* Data Direction Register, Port D bit 4, 0 = input, 1 = output */
#define    DDD3         3   /* Data Direction Register, Port D bit 3, 0 = input, 1 = output */
#define    DDD2         2   /* Data Direction Register, Port D bit 2, 0 = input, 1 = output */
#define    DDD1         1   /* Data Direction Register, Port D bit 1, 0 = input, 1 = output */
#define    DDD0         0   /* Data Direction Register, Port D bit 0, 0 = input, 1 = output */


/* PORT C register */
#define PINC	_SFR_IO8(0x13)  /* Input Pins, Port C */
#define DDRC	_SFR_IO8(0x14)  /* Data Direction Register, Port C */
#define PORTC	_SFR_IO8(0x15)  /* Data Register, Port C */

#define    PC7          7   /* Data Register - Port C bit 7 */
#define    PC6          6   /* Data Register - Port C bit 6 */
#define    PC5          5   /* Data Register - Port C bit 5 */
#define    PC4          4   /* Data Register - Port C bit 4 */
#define    PC3          3   /* Data Register - Port C bit 3 */
#define    PC2          2   /* Data Register - Port C bit 2 */
#define    PC1          1   /* Data Register - Port C bit 1 */
#define    PC0          0   /* Data Register - Port C bit 0 */

#define    DDC7         7   /* Data Direction Register, Port C bit 7, 0 = input, 1 = output */
#define    DDC6         6   /* Data Direction Register, Port C bit 6, 0 = input, 1 = output */
#define    DDC5         5   /* Data Direction Register, Port C bit 5, 0 = input, 1 = output */
#define    DDC4         4   /* Data Direction Register, Port C bit 4, 0 = input, 1 = output */
#define    DDC3         3   /* Data Direction Register, Port C bit 3, 0 = input, 1 = output */
#define    DDC2         2   /* Data Direction Register, Port C bit 2, 0 = input, 1 = output */
#define    DDC1         1   /* Data Direction Register, Port C bit 1, 0 = input, 1 = output */
#define    DDC0         0   /* Data Direction Register, Port C bit 0, 0 = input, 1 = output */


/* PORT B register */
#define PINB	_SFR_IO8(0x16)  /* Input Pins, Port B */
#define DDRB	_SFR_IO8(0x17)  /* Data Direction Register, Port B */
#define PORTB	_SFR_IO8(0x18)  /* Data Register, Port B */

#define    PB7          7   /* Data Register - Port B bit 7 */
#define    PB6          6   /* Data Register - Port B bit 6 */
#define    PB5          5   /* Data Register - Port B bit 5 */
#define    PB4          4   /* Data Register - Port B bit 4 */
#define    PB3          3   /* Data Register - Port B bit 3 */
#define    PB2          2   /* Data Register - Port B bit 2 */
#define    PB1          1   /* Data Register - Port B bit 1 */
#define    PB0          0   /* Data Register - Port B bit 0 */

#define    DDB7         7   /* Data Direction Register, Port B bit 7, 0 = input, 1 = output */
#define    DDB6         6   /* Data Direction Register, Port B bit 6, 0 = input, 1 = output */
#define    DDB5         5   /* Data Direction Register, Port B bit 5, 0 = input, 1 = output */
#define    DDB4         4   /* Data Direction Register, Port B bit 4, 0 = input, 1 = output */
#define    DDB3         3   /* Data Direction Register, Port B bit 3, 0 = input, 1 = output */
#define    DDB2         2   /* Data Direction Register, Port B bit 2, 0 = input, 1 = output */
#define    DDB1         1   /* Data Direction Register, Port B bit 1, 0 = input, 1 = output */
#define    DDB0         0   /* Data Direction Register, Port B bit 0, 0 = input, 1 = output */


/* PORT A register */
#define PINA	_SFR_IO8(0x19)  /* Input Pins, Port A */
#define DDRA	_SFR_IO8(0x1A)  /* Data Direction Register, Port A */
#define PORTA	_SFR_IO8(0x1B)  /* Data Register, Port A */

#define    PA7          7   /* Data Register - Port A bit 7 */
#define    PA6          6   /* Data Register - Port A bit 6 */
#define    PA5          5   /* Data Register - Port A bit 5 */
#define    PA4          4   /* Data Register - Port A bit 4 */
#define    PA3          3   /* Data Register - Port A bit 3 */
#define    PA2          2   /* Data Register - Port A bit 2 */
#define    PA1          1   /* Data Register - Port A bit 1 */
#define    PA0          0   /* Data Register - Port A bit 0 */

#define    DDA7         7   /* Data Direction Register, Port A bit 7, 0 = input, 1 = output */
#define    DDA6         6   /* Data Direction Register, Port A bit 6, 0 = input, 1 = output */
#define    DDA5         5   /* Data Direction Register, Port A bit 5, 0 = input, 1 = output */
#define    DDA4         4   /* Data Direction Register, Port A bit 4, 0 = input, 1 = output */
#define    DDA3         3   /* Data Direction Register, Port A bit 3, 0 = input, 1 = output */
#define    DDA2         2   /* Data Direction Register, Port A bit 2, 0 = input, 1 = output */
#define    DDA1         1   /* Data Direction Register, Port A bit 1, 0 = input, 1 = output */
#define    DDA0         0   /* Data Direction Register, Port A bit 0, 0 = input, 1 = output */


/*---------------------------------------------------------------------------*/
/* EEPROM Registers */

#define EECR	_SFR_IO8(0x1C)  /* EEPROM Control Register */

#define    EERIE        3       /* EEPROM Ready Interrupt Enable, 1 = enable */
#define    EEMWE        2       /* EEPROM Master Write Enable, 1 - enable EEWE bit */
#define    EEWE         1       /* EEPROM Write Enable, 1 = write EEPROM */
                                /*   must be done within 4 clocks of writing EEMWE bit */
#define    EERE         0       /* EEPROM Read Enable, 1 = read EEPROM */

#define EEDR	_SFR_IO8(0x1D)  /* EEPROM Data Register */
#define EEAR	_SFR_IO16(0x1E) /* EEPROM Address Register - 16 bit access */
#define EEARL	_SFR_IO8(0x1E)  /* EEPROM Address Register -  8 bit access (LSB) */
#define EEARH	_SFR_IO8(0x1F)  /* EEPROM Address Register -  8 bit access (MSB) */


/*---------------------------------------------------------------------------*/
/* Watchdog Register */

#define WDTCR	_SFR_IO8(0x21)  /* Watchdog Timer Control Register */

#define    WDCE         4       /* Watchdog Change Enable */
#define    WDE          3       /* Watchdog Enable */
#define    WDP2         2       /* Watchdog Timer Prescalar bit 2 */
#define    WDP1         1       /* Watchdog Timer Prescalar bit 1 */
#define    WDP0         0       /* Watchdog Timer Prescalar bit 0 */

#define WD_CHANGE_ENABLE        (_BV(WDCE) | _BV(WDE))
#define WD_TIMER_PRESCALE_16K   (0)
#define WD_TIMER_PRESCALE_32K   _BV(WDP0)
#define WD_TIMER_PRESCALE_64K   _BV(WDP1)
#define WD_TIMER_PRESCALE_128K  (_BV(WDP1) | _BV(WDP0))
#define WD_TIMER_PRESCALE_256K  _BV(WDP2)
#define WD_TIMER_PRESCALE_512K  (_BV(WDP2) | _BV(WDP0))
#define WD_TIMER_PRESCALE_1024K (_BV(WDP2) | _BV(WDP1))
#define WD_TIMER_PRESCALE_2048K (_BV(WDP2) | _BV(WDP1) | _BV(WDP0))


/*---------------------------------------------------------------------------*/
#define SFIOR   _SFR_IO8(0x30)  /* Special Function IO Register */

#define    XMBK         6       /* External Memory Bus Keeper Enable, 1 = enable */
#define    XMM2         5       /* External Memory High Mask - bit 2 */
#define    XMM1         4       /* External Memory High Mask - bit 1 */
#define    XMM0         3       /* External Memory High Mask - bit 0 */
#define    PUD          2       /* Pull-up Disable, 1 = disable */
#define    PSR10        0       /* Prescaler Reset Timer/Counter 1 and Timer/COunter 0, 1 = reset */


/*---------------------------------------------------------------------------*/
/* Timer/Counter 0 & 1 Registers */

#define ICR1	_SFR_IO16(0x24) /* Timer/Counter 1 Input Capture Register - 16 bit access */
#define ICR1L	_SFR_IO8(0x24)  /* Timer/Counter 1 Input Capture Register -  8 bit access (LSB) */
#define ICR1H	_SFR_IO8(0x25)  /* Timer/Counter 1 Input Capture Register -  8 bit access (MSB) */

#define OCR1B	_SFR_IO16(0x28) /* Timer/Counter1 Output Compare Register B - 16 bit access */
#define OCR1BL	_SFR_IO8(0x28)  /* Timer/Counter1 Output Compare Register B -  8 bit access (LSB) */
#define OCR1BH	_SFR_IO8(0x29)  /* Timer/Counter1 Output Compare Register B -  8 bit access (MSB) */

#define OCR1A	_SFR_IO16(0x2A) /* Timer/Counter1 Output Compare Register A - 16 bit access */
#define OCR1AL	_SFR_IO8(0x2A)  /* Timer/Counter1 Output Compare Register A -  8 bit access (LSB) */
#define OCR1AH	_SFR_IO8(0x2B)  /* Timer/Counter1 Output Compare Register A -  8 bit access (MSB) */

#define TCNT1	_SFR_IO16(0x2C) /* Timer/Counter 1 - 16 bit access */
#define TCNT1L	_SFR_IO8(0x2C)  /* Timer/Counter 1 -  8 bit access (LSB) */
#define TCNT1H	_SFR_IO8(0x2D)  /* Timer/Counter 1 -  8 bit access (MSB) */

#define TCCR1B	_SFR_IO8(0x2E)  /* Timer/Counter 1 Control and Status Register */

#define    ICNC1        7       /* Input capture Noise Canceler, 1 = activate */
#define    ICES1        6       /* Input capture Edge Select, 0 = falling edge, 1 = positive edge */
#define    WGM13        4       /* Waveform Generation Mode bit 3 */
#define    WGM12        3       /* Waveform Generation Mode bit 2 */
#define    CS12         2       /* Clock Select bit 2 */
#define    CS11         1       /* Clock Select bit 1 */
#define    CS10         0       /* Clock Select bit 0 */

#define TC1_CS_STOPPED  (0)                                     /* timer stopped */
#define TC1_CS_DIV_1    _BV(CS10)                               /* prescalar - divide by 1 */
#define TC1_CS_DIV_8    _BV(CS11)                               /* prescalar - divide by 8 */
#define TC1_CS_DIV_64   (_BV(CS11) | _BV(CS10))                 /* prescalar - divide by 64 */
#define TC1_CS_DIV_256  _BV(CS12)                               /* prescalar - divide by 256 */
#define TC1_CS_DIV_1024 (_BV(CS12) | _BV(CS10))                 /* prescalar - divide by 1024 */
#define TC1_CS_EXT_FE   (_BV(CS12) | _BV(CS11))                 /* external clock on T1 pin, falling edge */
#define TC1_CS_EXT_RE   (_BV(CS12) | _BV(CS11) | _BV(CS10))     /* external clock on T1 pin, rising  edge */


#define TCCR1A	_SFR_IO8(0x2F)  /* Timer/Counter 1 Control Register */

#define    COM1A1       7       /* Compare Ouput Mode for Channel A - bit 1 */
#define    COM1A0       6       /* Compare Ouput Mode for Channel A - bit 0 */
#define    COM1B1       5       /* Compare Ouput Mode for Channel B - bit 1 */
#define    COM1B0       4       /* Compare Ouput Mode for Channel B - bit 0 */
#define    FOC1A        3       /* Force Output Compare for Channel A */
#define    FOC1B        2       /* Force Output Compare for Channel B */
#define    WGM11        1       /* Waveform Generation Mode bit 1 */
#define    WGM10        0       /* Waveform Generation Mode bit 0 */


#define OCR0    _SFR_IO8(0x31)  /* Timer/Counter 0 Output Compare Register */

#define TCNT0	_SFR_IO8(0x32)  /* Timer/Counter 0 */

#define TCCR0	_SFR_IO8(0x33)  /* Timer/Counter 0 Control Register */

#define    FOC0         7       /* Force Output Compare, 1 = force */
#define    WGM00        6       /* Waveform Generation Mode - bit 0 */
#define    COM01        5       /* Compare Match Output Mode - bit 1 */
#define    COM00        4       /* Compare Match Output Mode - bit 0 */
#define    WGM01        3       /* Waveform Generation Mode - bit 1 */
#define    CS02         2       /* Clock Select - bit 2 */
#define    CS01         1       /* Clock Select - bit 1 */
#define    CS00         0       /* Clock Select - bit 0 */


#define TIFR	_SFR_IO8(0x38)  /* Timer/Counter Interrupt Flag register */

#define    TOV1         7       /* Timer/Counter 1 Overflow Flag, 1 = overflow */
#define    OCF1A        6       /* Timer/COunter 1 Output Compare A Match Flag, 1 = match */
#define    OCF1B        5       /* Timer/COunter 1 Output Compare B Match Flag, 1 = match */
#define    ICF1         3       /* Timer/Counter 1 Input Capture Flag, 1 = input captured */
#define    TOV0         1       /* Timer/Counter 0 Overflow Flag, 1 = overflow */
#define    OCF0         0       /* Output Compare Flag 0, 1 = match */


#define TIMSK	_SFR_IO8(0x39)  /* Timer/Counter Interrupt MaSK register */

#define    TOIE1        7       /* Timer/Counter1 Overflow Interrupt Enable */
#define    OCIE1A       6       /* Timer/Counter1 Output Compare A Match Interrupt Enable */
#define    OCIE1B       5       /* Timer/Counter1 Output Compare B Match Interrupt Enable */
#define    TICIE1       3       /* Timer/Counter1 Input Capture Interrupt Enable */
#define    TOIE0        1       /* Timer/Counter0 Overflow Interrupt Enable */
#define    OCIE0        0       /* Timer/Counter0 Output Compare Match Interrupt Enable */

#define TC1_WG_MODE_NORMAL      0   /* count up, cycle 0x0000 to 0xFFFF */
#define TC1_WG_MODE_CTC_OCR1A   4   /* count up, cycle 0x0000 to OCR1A */

#define TC1_SET_WG_MODE         
/*---------------------------------------------------------------------------*/
/* MCU Registers */

#define MCUCSR  _SFR_IO8(0x34)  /* MCU Control and Status Register */

#define    SM2          5       /* Sleep Mode Select - bit 2 */
#define    WDRF         3       /* Watchdog Reset Flag,  1 = watchdog reset occurred */
#define    BORF         2       /* Brown-out Reset Flag, 1 = brown-out reset occurred */
#define    EXTRF        1       /* External Reset Flag,  1 = external reset occurred */
#define    PORF         0       /* Power-on Reset Flag,  1 = power-on reset occurred */


#define MCUCR   _SFR_IO8(0x35)  /* MCU Control Register */

#define    SRE          7       /* External SRAM/XMEM Enable, 1 = enable */
#define    SRW10        6       /* Upper Sector Wait State Select - bit 0 */ 
#define    SE           5       /* Sleep Enable, 1 = enable */
#define    SM1          4       /* Sleep Mode Select - bit 1 */
#define    ISC11        3       /* Interrupt Sense Control 1 - bit 1 */
#define    ISC10        2       /* Interrupt Sense Control 1 - bit 0 */
#define    ISC01        1       /* Interrupt Sense Control 0 - bit 1 */
#define    ISC00        0       /* Interrupt Sense Control 0 - bit 0 */


#define EMCUCR  _SFR_IO8(0x36)  /* Extended MCU Control Register */

#define    SM0          7       /* Sleep Mode Select - bit 0 */
#define    SRL2         6       /* Wait State Sector Limit - bit 2 */
#define    SRL1         5       /* Wait State Sector Limit - bit 1 */
#define    SRL0         4       /* Wait State Sector Limit - bit 0 */
#define    SRW01        3       /* Lower Sector Wait State Select - bit 1 */
#define    SRW00        2       /* Lower Sector Wait State Select - bit 0 */
#define    SRW11        1       /* Upper Sector Wait State Select - bit 1 */
#define    ISC2         0       /* Interrupt Sense Control 2 */

#define SLEEP_MODE_IDLE         0
#define SLEEP_MODE_POWER_DOWN   2
#define SLEEP_MODE_STANDBY      6


#define GIFR    _SFR_IO8(0x3A)  /* General Interrupt Flag Register */

#define    INTF1        7       /* External Interrupt Flag 1, 1 = INT1 pin interrupt occurred */
#define    INTF0        6       /* External Interrupt Flag 0, 1 = INT0 pin interrupt occurred */
#define    INTF2        5       /* External Interrupt Flag 2, 1 = INT2 pin interrupt occurred */


#define GICR    _SFR_IO8(0x3B)  /* General Interrupt Control Register */

#define    INT1         7       /* External Interrupt Request 1 Enable, 1 = enable INT1 pin */
#define    INT0         6       /* External Interrupt Request 0 Enable, 1 = enable INT0 pin */
#define    INT2         5       /* External Interrupt Request 2 Enable, 1 = enable INT2 pin */
#define    IVSEL        1       /* Interrupt Vector Select, 1 = relocate vectors to FLASH boot loader section */
#define    IVCE         0       /* Interrupt Vector Change Enable, 1 = enable IVSEL bit */

/* 0x3D..0x3E SP */

/* 0x3F SREG */

/*---------------------------------------------------------------------------*/
/* FLASH Read/Write Control Registers */

#define SPMCR   _SFR_IO8(0x37)  /* Store Program Memory Control Register */

#define    SPMIE        7       /* SPM Interrupt Enable, 1 = enable */
#define    RWWSB        6       /* Read While Write Section Busy, 1 = busy */
#define    RWWSRE       4       /* Read While Write Section Read Enable, 1 = enable */
#define    BLBSET       3       /* Boot Lock Bit Set, 1 = set boot lock bits */
#define    PGWRT        2       /* Page Write, 1 = execute page write */
#define    PGERS        1       /* Page Erase, 1 = execute page erase */
#define    SPMEN        0       /* Store Program Memory Enable, 1 = enable */


/*---------------------------------------------------------------------------*/
/* Interrupt vectors */

/* External Interrupt Request 0 */
#define INT0_vect			    _VECTOR(1)
#define SIG_INTERRUPT0			_VECTOR(1)

/* External Interrupt Request 1 */
#define INT1_vect			    _VECTOR(2)
#define SIG_INTERRUPT1			_VECTOR(2)

/* Timer/Counter1 Capture Event */
#define TIMER1_CAPT_vect		_VECTOR(3)
#define SIG_INPUT_CAPTURE1		_VECTOR(3)

/* Timer/Counter1 Compare Match A */
#define TIMER1_COMPA_vect		_VECTOR(4)
#define SIG_OUTPUT_COMPARE1A	_VECTOR(4)

/* Timer/Counter1 Compare MatchB */
#define TIMER1_COMPB_vect		_VECTOR(5)
#define SIG_OUTPUT_COMPARE1B	_VECTOR(5)

/* Timer/Counter1 Overflow */
#define TIMER1_OVF_vect			_VECTOR(6)
#define SIG_OVERFLOW1			_VECTOR(6)

/* Timer/Counter0 Overflow */
#define TIMER0_OVF_vect			_VECTOR(7)
#define SIG_OVERFLOW0			_VECTOR(7)

/* Serial Transfer Complete */
#define SPI_STC_vect			_VECTOR(8)
#define SIG_SPI				    _VECTOR(8)

/* UART, Rx Complete */
#define USART_RX_vect           _VECTOR(9)
#define UART_RX_vect            _VECTOR(9) /* For compatability only */
#define SIG_UART_RECV           _VECTOR(9) /* For compatability only */

/* UART Data Register Empty */
#define USART_UDRE_vect         _VECTOR(10)
#define UART_UDRE_vect          _VECTOR(10) /* For compatability only */
#define SIG_UART_DATA           _VECTOR(10) /* For compatability only */

/* UART, Tx Complete */
#define USART_TX_vect           _VECTOR(11)
#define UART_TX_vect            _VECTOR(11) /* For compatability only */
#define SIG_UART_TRANS          _VECTOR(11) /* For compatability only */

/* Analog Comparator */
#define ANA_COMP_vect			_VECTOR(12)
#define SIG_COMPARATOR			_VECTOR(12)

/* External Interrupt Request 2 */
#define INT2_vect			    _VECTOR(13)
#define SIG_INTERRUPT2			_VECTOR(13)

/* Timer 0 Compare Match */
#define TIMER0_COMP_vect		_VECTOR(14)
#define SIG_OUTPUT_COMPARE0		_VECTOR(14)

/* EEPROM Ready */
#define EE_RDY_vect			    _VECTOR(15)
#define SIG_EEPROM_READY		_VECTOR(15)

/* Store Program Memory Ready */
#define SPM_RDY_vect			_VECTOR(16)
#define SIG_SPM_READY			_VECTOR(16)

#define _VECTORS_SIZE 34


/*---------------------------------------------------------------------------*/
/* Constants */
#define    SPM_PAGESIZE 64
#define    RAMEND       0x25F    /* Last On-Chip SRAM Location */
#define    XRAMEND      0xFFFF
#define    E2END        0x1FF
#define    FLASHEND     0x1FFF


/*---------------------------------------------------------------------------*/
/* Fuses */

#define FUSE_MEMORY_SIZE 2

/* Low Fuse Byte */
#define CKSEL0      ~_BV(0)
#define CKSEL1      ~_BV(1)
#define CKSEL2      ~_BV(2)
#define CKSEL3      ~_BV(3)
#define SUT0        ~_BV(4)
#define SUT1        ~_BV(5)
#define BODEN       ~_BV(6)
#define BODLEVEL    ~_BV(7)
#define LFUSE_DEFAULT (CKSEL1 & CKSEL2 & CKSEL3 & SUT0)

/* High Fuse Byte */
#define BOOTRST     ~_BV(0)
#define BOOTSZ0     ~_BV(1)
#define BOOTSZ1     ~_BV(2)
#define EESAVE      ~_BV(3)
#define CKOPT       ~_BV(4)
#define SPIEN       ~_BV(5)
#define WDTON       ~_BV(6)
#define S8515C      ~_BV(7)
#define HFUSE_DEFAULT (BOOTSZ0 & BOOTSZ1 & SPIEN)


/* Lock Bits */
#define __LOCK_BITS_EXIST
#define __BOOT_LOCK_BITS_0_EXIST
#define __BOOT_LOCK_BITS_1_EXIST 

#endif  /* _AVR_ATMEGA8515_H_ */
