/*-----------------------------------------------------------------------------
 Writing Portable Device Drivers
   Filename:    main.c
   Filetype:    C source file
   Author:      Ray Hurst
   Created:     Mar 15, 2008
   Edited:      Apr 03, 2008

   Description:

   LAB 4
   Part 1:
   Write a driver to cycle the LEDs from 0 to 7 with a delay beteen cycles.
   If LED 7 is lit then the next LED lit will be LED 0.
   If the corresponding switch (0 to 7) is closed then reset back to LED 0.
   Delay controlled by a timer/counter ISR.

   Part2:
   Activate the USART with the settings 9600-8-N-1.
   Accept input from the USART Receiver and echo each character (except ctlr-z and ctrl-y)
   to the USART transmitter.
   A ctrl-z character us used to control storing of data to the EEPROM.
   A ctrl-y character is used to playback the contents of the EEPROM.

   USART operation is as follows:

   Record   Character   Task
   Mode     Received

     X      not         Echo character to USART TXC
            ctrl-x or
            ctrl-z or
            ctrl-y

     0      ctrl-z      Clear EEPROM contents and set Recode Mode to 1.
                        Record mode = 1 starts recording USART data to EEPROM

     1      ctrl-z      Set Record Mode to 0. Stops recording USART data to EEPROM.

     X      ctrl-y      playback EEPROM contents to the USART

     X      ctrl-x      display contents of the data SRAM



   STK500 Setup:
   PORT A - 8 switches used as input
   PORT B - 8 LEDs     used as output
            if LED6 is solid on - ABORTED
   Timer\Counter 1 used as delay generator
   USART set to Asych normal 9600-8-N-1.
*/

#include "at_mega_8515.h"

#define DEBUG_ENA_PORTD	0
#define DEBUG_EEPROM 	1
#define DEBUG_TIMER		0
#define DEBUG_USART		1

/*---------------------------------------------------------------------------*/
/* Function decalarations */

void    crlf (void );
void    displayData_formatted( uint16_t index, uint8_t data );
uint8_t bin2AsciiHex( uint8_t byte );
void    io_printString( const char *s );
void    io_putchar ( char c );
void    io_putHex( char c );
void    io_puts ( const char *s );
void    usart_putchar ( char c );


/*---------------------------------------------------------------------------*/
/* Constants */

#define F_CPU           3686400UL   /* CPU frequency (Hz) */

#define DELAY_COUNT     4           /* delay count - 18 msec resolution */

#define FALSE           0
#define TRUE            1

#define	BIT7            7
#define	BIT6            6
#define	BIT5            5
#define	BIT4            4
#define	BIT3            3
#define	BIT2            2
#define	BIT1            1
#define	BIT0            0

#define MAX_LEDS        8
#define MAX_SWITCHES    8

/* LED definitions */
#define ALL_LEDS_OFF    0xFF
#define LED_0_ON        ~_BV(BIT0)
#define LED_2_ON        ~_BV(BIT2)
#define LED_4_ON        ~_BV(BIT4)
#define	LED_6_ON        ~_BV(BIT6)
#define	LED_7_ON        ~_BV(BIT7)

#define ABORTED         LED_6_ON    /* LED to light up when program aborts */

/* USART definitions */
#define USART_BAUDRATE              9600
#define BAUD_PRESCALE_ASYNCH_NORMAL (((F_CPU / (USART_BAUDRATE * 16UL))) - 1 )

#define USART_RXC_FULL              ( 0 != (UCSRA & _BV(RXC)) )
#define USART_TXC_EMPTY             ( 0 != (UCSRA & _BV(UDRE)) )

/* ASCII control codes */
#define CTRL_X          0x18        /* used to display data SRAM */
#define CTRL_Y          0x19        /* used to playback stored data */
#define CTRL_Z          0x1A        /* used to enable/disable record mode */


/*---------------------------------------------------------------------------*/
/* EEPROM data constants */

#define EEPROM_ADDRESS_SIGNATURE    0
#define EEPROM_DATA_SIGNATURE       0x5A

#define EEPROM_ADDRESS_COUNT        1
#define EEPROM_DATA_COUNT_MAX       254

#define EEPROM_ADDRESS_DATA         2

typedef struct 
{
    uint8_t    signature;   /* EEPROM data valid signature */
    uint8_t    count;       /* number of bytes recorded */
} EEPROM_HEADER, *P_EEPROM_HEADER;;


/*---------------------------------------------------------------------------*/
/* Hardware Initialization constants  */

#define INIT_TCCR1A         0x00
#define INIT_TCCR1B         TC1_CS_DIV_1

#define INIT_DDRA_DIR       0x00        /* port A programmed as input  port */
#define INIT_DDRB_DIR       0xFF        /* port B programmed as output port */

#if (DEBUG_ENA_PORTD)
  #define INIT_DDRD_DIR     (_BV(DDD2) | _BV(DDD4) | _BV(DDD6))
#endif

#define INIT_PORTD          0x00


/*----------------------------------------------------------------------------*/
/* Global data */

#define DATA_MEMORY_SIZE    512

#define BUFFER_SIZE     128
uint8_t dataBuffer[BUFFER_SIZE];
uint8_t dataIndex_read;
uint8_t dataIndex_write;

#define INCREMENT_DATA_INDEX_READ   dataIndex_read  = (dataIndex_read + 1)  % BUFFER_SIZE
#define INCREMENT_DATA_INDEX_WRITE  dataIndex_write = (dataIndex_write + 1) % BUFFER_SIZE

#define RECORD_OFF  0
#define RECORD_ON   1
uint8_t recordMode = RECORD_OFF;

EEPROM_HEADER eePromHeader;


/*---------------------------------------------------------------------------*/
/* Abort routine */

void
weAreDead( void )
{
    cli();              /* disable global interrupts */
    PORTD = ABORTED;    /* set PORTD bit 6 - ABORTED */
    while(1);           /* loop forever */
}


/*---------------------------------------------------------------------------*/
/* EEPROM read/write routines */

/*
  Function    : EEPROM_read
  Description : read a byte from address in EEPROM
*/
uint8_t
EEPROM_read( uint16_t uiAddress )
{
    /* wait for completion from previous write */
    while( EECR & _BV(EEWE) )
        ;

    /* setup address register */
    EEAR = uiAddress;

    cli();	/* disable global interrupts */
    /* start the EEPROM read process */
    EECR |= _BV(EERE);
    /* return the data from the data register */
    sei();	/* enable global interrupts */
    return EEDR;
}


/*
  Function    : EEPROM_write
  Description : write a byte to address in EEPROM (8.5msec program time)
*/
void
EEPROM_write( uint16_t uiAddress, uint8_t ucData )
{
    /* wait for completion from previous write */
    while( EECR & _BV(EEWE) )
        ;

    /* setup address and data registers */
    EEAR = uiAddress;
    EEDR = ucData;

    cli();	/* disable global interrupts */
    /* start the EEPROM write process */
    EECR |= _BV(EEMWE);
    /* write the data to EEPROM */
    EECR |= _BV(EEWE);
    sei();	/* enable global interrupts */
}


/*
  Function    : EEPROM_displayHeader
  Description : display the EEPROM header
*/
void
EEPROM_displayHeader( P_EEPROM_HEADER pEEpromHeader )
{
    crlf();
    io_printString( "Signature: 0x" );
    io_putHex( eePromHeader.signature );
    crlf();
    io_printString( "Count: 0x" );
    io_putHex( eePromHeader.count );
    crlf();
}


/*
  Function    : EEPROM_verifyHeader
  Description : read and validate the EEPROM header
*/
uint8_t
EEPROM_verifyHeader( P_EEPROM_HEADER pEEpromHeader )
{
    uint8_t dataValid = FALSE;  /* assume the EEPROM data is invalid */

    /* read signature bytes */
    pEEpromHeader->signature  = EEPROM_read( EEPROM_ADDRESS_SIGNATURE );
    pEEpromHeader->count      = EEPROM_read( EEPROM_ADDRESS_COUNT );

    if( EEPROM_DATA_SIGNATURE == pEEpromHeader->signature ) 
        dataValid = TRUE;
    return dataValid;
}


/*
  Function    : EEPROM_initialize
  Description : write EEPROM header
*/
void
EEPROM_initializeHeader( void )
{
    EEPROM_write( EEPROM_ADDRESS_SIGNATURE, EEPROM_DATA_SIGNATURE );
    EEPROM_write( EEPROM_ADDRESS_COUNT,     0 );
}


/*
  Function    : EEPROM_playback
  Description : send each byte in the EEPROM to the USART transmitter
*/
void
EEPROM_playback( void )
{
    if( TRUE == EEPROM_verifyHeader(&eePromHeader) ) {
        uint8_t i, j, data;

        EEPROM_displayHeader( &eePromHeader );

        for ( i = 0, j= 0; i < eePromHeader.count; i++ ) 
        {
            data = EEPROM_read( EEPROM_ADDRESS_DATA + i );
            displayData_formatted( j, data );
            j++;
        }
        crlf();
    }
    else 
    {
        crlf();
        io_puts( "EEPROM invalid!" );
    }
}


/*
  Function    : EEPROM_record
  Description : store USART data received into EEPROM
*/
void
EEPROM_record( void )
{
    if( TRUE == EEPROM_verifyHeader(&eePromHeader) ) {
        EEPROM_displayHeader( &eePromHeader );

        while( dataIndex_write != dataIndex_read ) 
        {
            if ( EEPROM_DATA_COUNT_MAX == eePromHeader.count ) 
            {
                crlf();
                io_puts( "EEPROM full!" );
                dataIndex_write = dataIndex_read;   /* discard remaining characters */
                break;
            }
            else
            {

                EEPROM_write( EEPROM_ADDRESS_DATA + eePromHeader.count, dataBuffer[dataIndex_write] );
                eePromHeader.count++;
                INCREMENT_DATA_INDEX_WRITE;
            }
        }
        /* update EEPROM data log counter */
        EEPROM_write( EEPROM_ADDRESS_COUNT, eePromHeader.count );
    }
    else 
    {
        crlf();
        io_puts( "EEPROM invalid!" );
    }
}


/*---------------------------------------------------------------------------*/
/* Utility routines */

/*
    Function:   crlf
    Description:
        Output CR, LF characters to standard output
*/
void
crlf (void )
{
    io_putchar( '\r' );
    io_putchar( '\n' );
}


/*
    Function:	displayData_formatted
    Description:
        Display data as ASCII hex, neatly formatted 16 bytes per line
*/
void
displayData_formatted ( uint16_t index, uint8_t data )
{
    if ( 0 == (index % 16) ) 
    {
        crlf();
        io_putHex( index >> 8 );
        io_putHex( index );
        io_putchar( ':' );
    }
    io_putchar( ' ' );
    io_putHex( data );
}


/*
    Function:	dumpDataMemory
    Description:
        Display the data memory (512 bytes)
*/
void
dumpDataMemory( void )
{
    uint16_t i;

    uint8_t *dataPtr = (uint8_t *)0x60;

    crlf();
    io_puts( "DM Dump:");
    for( i = 0; i < 512; i++ )
        displayData_formatted( i, *dataPtr++ );
}


/*
    Function:	bin2AsciiHex
    Description:
        This function converts a the lower 4 bits of a binary byte into an ASCII hex character
*/
uint8_t
bin2AsciiHex( uint8_t byte )
{
    byte = byte & 0x0F;
    if( byte > 9 )
        byte += 7;
    byte += '0';
    return byte;
}


/*---------------------------------------------------------------------------*/
/* USART I/O routines */

/*
    Function:   usart_putchar()
    Description:
        This function writes a single byte to the USART tranmit data register.
*/

void
usart_putchar ( char c )
{
    /* wait for empty transmit buffer */
    while( !( UCSRA & _BV(UDRE)) ) ;
    UDR = c;
}


/*---------------------------------------------------------------------------*/
/* I/O routines */

/*
    Function:   io_printString()
    Description:
        Output a string pointed to by s to the standard output device.
        The terminating null byte is not written.
*/
void
io_printString( const char *s )
{
    while( '\0' != *s )
        io_putchar( *s++ );
}


/*
    Function:   io_putchar()
    Description:
        Output a single character to the standard output device.
*/

void
io_putchar ( char c )
{
    usart_putchar( c );
}


/*
    Function:   io_puts()
    Description:
        Output a string pointed to by s followed by a newline to the standard output device.
        The terminating null byte is not written.
*/
void
io_puts ( const char *s )
{
	io_printString( s );
	crlf();
}


/*
	Function:	io_putHex()
	Description:
		Convert a byte value into two ASCII hex characters and output them to the standard output device.
*/
void
io_putHex( char c )
{
	io_putchar( bin2AsciiHex(c >> 4) );
	io_putchar( bin2AsciiHex(c) );
}


/*---------------------------------------------------------------------------*/
/* Interrupt Sevice Routines (ISRs) */

/* Timer/Counter 1 Overflow Interrupt handler - 20msec interrupt */
/* Configured as an interruptable ISR */
ISR (TIMER1_OVF_vect, ISR_NOBLOCK)
{
	static uint8_t i = 0;
	static uint8_t delayCount = DELAY_COUNT;

 #if (DEBUG_TIMER)
 	PORTD = PIND ^ _BV(6);				/* toggle PORTD pin 6 */
 #endif

	if( 0 == --delayCount ) {
		delayCount   = DELAY_COUNT;

        PORTB = ~_BV(i);
        i = (i + 1) % 8;				/* cycle to the next LED */

		/* is the switch coresponding to the lit LED closed? */
        if( 0 == (PINA & _BV(i)) )
            i = 0;						/* here, the switch is closed */
	}
}

/* USART receiver full interrupt */
ISR(USART_RX_vect)
{
    /* store the USART data and exit */
	dataBuffer[dataIndex_read] = UDR;
	INCREMENT_DATA_INDEX_READ;          	/* note: no buffer overflow detection */
}


/*---------------------------------------------------------------------------*/
/* main initialization routine */

void
init_system(void)
{
	/* initialize global data */
	dataIndex_read = dataIndex_write = 0;

	/* initialize hardware */
  #if (DEBUG_ENA_PORTD)
	PORTD = INIT_PORTD;
	DDRD  = INIT_DDRD_DIR;
  #endif

	PORTB = ALL_LEDS_OFF;	/* initialize PORTB */
	DDRB  = INIT_DDRB_DIR;	/* set PORTB direction register */

	DDRA  = INIT_DDRA_DIR;	/* set PORTA direction register */

	/* initialize timer/counter 1 */	
    TCCR1A = INIT_TCCR1A;
    TCCR1B = INIT_TCCR1B;
    TIMSK = _BV (TOIE1);

	/* initialize USART */
    UBRRL = BAUD_PRESCALE_ASYNCH_NORMAL;
    UBRRH = BAUD_PRESCALE_ASYNCH_NORMAL >> 8;

	UCSRC = USART_REG_SELECT_UCSRC         | \
	        USART_MODE_SELECT_ASYNCHRONOUS | \
	        USART_PARITY_MODE_NONE         | \
            USART_STOP_BITS_1              | \
            USART_CHAR_SIZE_BITS_8_OR_9;

    UCSRB = _BV(RXCIE) | _BV(RXEN) | _BV(TXEN);  /* enable USART transmitter / receiver */

    sei();	/* enable global interrupts */
}


/* main routine */
int
main( void )
{
    init_system();

    crlf();
    io_puts( "USART v0.30" );    /* if we see this, we did OK */
    crlf();
    io_puts( "Ctrl-x - dump data memory" );
    io_puts( "Ctrl-z - start/stop data recording" );
    io_puts( "Ctrl-y - playback recorded data" );

    /* check if EEPROM is initialized */
    if( FALSE == EEPROM_verifyHeader(&eePromHeader) ) {
        /* EEPROM not initialized here */
        crlf();
        io_printString( "EEPROM Init ... " );
        EEPROM_initializeHeader();
        if( FALSE == EEPROM_verifyHeader(&eePromHeader) )
            io_puts( "FAIL" );
        else
            io_puts( "OK" );
        crlf();
    }

    while(1) {
  #if (DEBUG_USART)
        volatile uint16_t i = 0;

	     if( 0 == i++ ) {
		     crlf();
		 	io_printString( "Main loop: RI = ");
			io_putHex( dataIndex_read );
			io_printString( "  WI = " );
			io_putHex( dataIndex_write );
			crlf();
		}
  #endif
		/* process characters in the buffer */
		if(  dataIndex_write != dataIndex_read ) {
            /* here, we have data from the USART in the buffer */
			switch( dataBuffer[dataIndex_write] )
			{
			  case CTRL_X:
				INCREMENT_DATA_INDEX_WRITE;
                dumpDataMemory();
			    break;

			  case CTRL_Y:
			    crlf();
				io_puts( "Playback data:");
			    EEPROM_playback();      /* diplay EEPROM contents */
				INCREMENT_DATA_INDEX_WRITE;
			    break;

              case CTRL_Z:
                if (  RECORD_OFF == recordMode ) 
                {
                    /* here, recording is disabled, enable recording */
					recordMode = RECORD_ON;
					crlf();
					io_puts( "RecMode : ON" );
                    /* initialize the EEPROM */
                    EEPROM_initializeHeader();
                }
                else 
                {
                    // here, recording is enabled and we need to disable it */
                    recordMode = RECORD_OFF;
					crlf();
					io_puts( "RecMode : OFF" );
                }
				INCREMENT_DATA_INDEX_WRITE;
			    break;

			  default:
                if (  RECORD_ON == recordMode )
                {
    			    EEPROM_record();
                }
				else 
				{
                    io_putchar( dataBuffer[dataIndex_write] );
    				INCREMENT_DATA_INDEX_WRITE;
				}
                break;
			}
		}
	}
    return 0;
}

/* End of file main.c */
